/**
 * $RCSfile$
 * version $Revision$
 * created 07.06.2008 11:32:39 by kunina
 * last modified $Date$ by $Author$
 * (C) ООО Крипто-Про 2004-2008.
 *
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован 
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package Applet_samples.AppletNew;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.KeyStore.KeyStoreConfig;
import ru.CryptoPro.JCP.tools.Array;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyAgreement;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

/**
 * Класс для работы с провайдером.
 *
 * @author Copyright 2004-2008 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class Processor {
/**
 * Хранилище
 */
private static KeyStore store;
/**
 * Тип хранилища (HDImageStore, FloppyStore и т.д.)
 */
private static String storeName;
/**
 * Ключ
 */
private static Key key;
/**
 * Имя контейнера
 */
private static String alias;
/**
 * Пароль на контейнер
 */
private static char[] keyPass;
/**
 * Файл для подписи/шифрования
 */
private static File file;
/**
 * Сертификат второй стороны
 */
private static Certificate cert;

 /**/
public Processor() {
}

/**
 * Проверка установлен ли провайдер
 *
 * @return true/false
 */
protected static boolean checkJCP() {
    boolean ret = false;
    final Provider[] provs = Security.getProviders();
    for (int i = 0; i < provs.length; i++) {
        if ("JCP".equals(provs[i].getName())) {
            ret = true;
            break;
        }
    }
    return ret;
}

/**
 * Простейшая проверка работы провайдера
 *
 * @return true/false
 */
protected static boolean checkJCPwork() {
    boolean ret;
    try {
        Signature.getInstance("GOST3411withGOST3410EL");
        ret = true;
    } catch (NoSuchAlgorithmException e) {
        ret = false;
    }
    return ret;
}

/**
 * Установка типа хранилища
 *
 * @param keyStoreName тип
 */
protected static void setKeyStoreName(String keyStoreName) {
    storeName = keyStoreName;
}

/**
 * Получение типа хранилища
 *
 * @return тип
 */
protected static String getKeyStoreName() {
    return storeName;
}

/**
 * Установка имени контейнера
 *
 * @param keyAlias имя контейнера
 */
protected static void setKeyAlias(String keyAlias) {
    alias = keyAlias;
}

/**
 * Получение имени контейнера
 *
 * @return имя контейнера
 */
protected static String getKeyAlias() {
    return alias;
}

/**
 * Установка пароля на контейнер
 *
 * @param password пароль на контейнер
 */
protected static void setKeyPassword(char[] password) {
    keyPass = password;
}

/**
 * Получение имен типов хранилищ JCP.
 *
 * @return список имен
 */
protected static String[] getStores() {
    final List strs = KeyStoreConfig.getNames();
    final String[] stores = new String[strs.size()];
    for (int i = 0; i < strs.size(); i++) {
        stores[i] = (String) strs.get(i);
    }
    return stores;
}

/**
 * получение списка имен контенеров в данном хранилище
 *
 * @param storeNam тип хранилища
 * @return список имен
 */
protected static Enumeration getAliases(String storeNam) {
    Enumeration aliases = null;
    try {
        final KeyStore ks = KeyStore.getInstance(storeNam);
        store = ks;
        ks.load(null, null);
        aliases = ks.aliases();
    } catch (KeyStoreException e) {
        e.printStackTrace();
    } catch (NoSuchAlgorithmException e) {
        e.printStackTrace();
    } catch (IOException e) {
        e.printStackTrace();
    } catch (CertificateException e) {
        e.printStackTrace();
    }
    return aliases;
}

/**
 * Чтение ключа
 *
 * @return true/false
 */
protected static boolean readKey() {
    boolean ret = false;
    try {
        key = store.getKey(alias, keyPass);
        ret = true;
    } catch (NoSuchAlgorithmException e) {
        e.printStackTrace();
    } catch (UnrecoverableKeyException e) {
        e.printStackTrace();
    } catch (KeyStoreException e) {
        e.printStackTrace();
    }
    return ret;
}

/**
 * Установка пути к исходному файлу
 *
 * @param pfile файл
 */
protected static void setFile(File pfile) {
    file = pfile;
}

/**
 * Задан ли файл
 *
 * @return true/false
 */
protected static boolean isSetFile() {
    return file != null;
}

/**
 * Установка сертификата
 *
 * @param certificate файл
 */
protected static void setCert(Certificate certificate) {
    cert = certificate;
}

/**
 * Задан ли сертификат
 *
 * @return true/false
 */
protected static boolean isSetCert() {
    return cert != null;
}

/**
 * Получение сертификата из файла
 *
 * @param certFile файл сертификата
 * @return сертификат
 */
protected static List genCertFromFile(File certFile) {
    final List ret = new Vector(2);
    String err = null;
    Certificate certif = null;
    try {
        final CertificateFactory cf = CertificateFactory.getInstance("X509");
        if (certFile != null)
            certif = cf.generateCertificate(new FileInputStream(certFile));
    } catch (CertificateException e) {
        err = "Can't generate certificate";
    } catch (FileNotFoundException e) {
        err = e.toString();
    }
    ret.add(0, err);
    ret.add(1, certif);
    return ret;
}

/**
 * Подпись
 *
 * @return подпись
 */
protected static List sign() {
    final List ret = new Vector(2);
    String err = null;
    byte[] signature = null;
    if (file != null)
        try {
            final Signature sign =
                    Signature.getInstance("GOST3411withGOST3410EL");
            sign.initSign((PrivateKey) key);
            sign.update(Array.readFile(file));
            signature = sign.sign();
        } catch (NoSuchAlgorithmException e) {
            err = e.toString();
        } catch (SignatureException e) {
            err = e.toString();
        } catch (InvalidKeyException e) {
            err = e.toString();
        } catch (IOException e) {
            err = e.toString();
        }
    ret.add(0, err);
    ret.add(1, signature);
    return ret;
}

/**
 * Проверка подписи
 *
 * @param fileWithSign файл с подписью
 * @return /
 */
public static List signVerify(File fileWithSign) {
    final List ret = new Vector(2);
    String err = null;
    boolean ver = false;
    if (file != null && fileWithSign != null)
        try {
            final Signature sig =
                    Signature.getInstance("GOST3411withGOST3410EL");
            sig.initVerify(cert.getPublicKey());
            sig.update(Array.readFile(file));
            ver = sig.verify(Array.readFile(fileWithSign));
        } catch (NoSuchAlgorithmException e) {
            err = e.toString();
        } catch (InvalidKeyException e) {
            err = e.toString();
        } catch (IOException e) {
            err = e.toString();
        } catch (SignatureException e) {
            err = e.toString();
        }
    ret.add(0, err);
    ret.add(1, String.valueOf(ver));
    return ret;
}

/**
 * Шифрование
 *
 * @param mode режим
 * @param isENCRYPT encrypt/decrypt
 * @return encrypted/decrypted text
 */
public static List crypt(String mode, boolean isENCRYPT) {
    final List ret = new Vector(2);
    String err = null;
    final int mod;
    byte[] crypted = null;
    if (isENCRYPT) mod = Cipher.ENCRYPT_MODE;
    else mod = Cipher.DECRYPT_MODE;
    try {
        final KeyAgreement keyAgree = KeyAgreement.getInstance(JCP.GOST_EL_DH_NAME);
        //todo iv
        final AlgorithmParameterSpec iv =
                new IvParameterSpec(new byte[]{1, 2, 3, 4, 5, 6, 7, 8});
        keyAgree.init(key, iv, null);//iv
        keyAgree.doPhase(cert.getPublicKey(), true);
        final SecretKey secret = keyAgree.generateSecret("GOST28147");
        final Cipher cipher = Cipher.getInstance(mode);
        cipher.init(mod, secret, iv);//iv
        crypted = cipher.doFinal(Array.readFile(file));
    } catch (NoSuchAlgorithmException e) {
        err = e.toString();
    } catch (NoSuchPaddingException e) {
        err = e.toString();
    } catch (InvalidKeyException e) {
        err = e.toString();
    } catch (BadPaddingException e) {
        err = e.toString();
    } catch (IOException e) {
        err = e.toString();
    } catch (IllegalBlockSizeException e) {
        err = e.toString();
    } catch (InvalidAlgorithmParameterException e) {
        err = e.toString();
    }
    ret.add(0, err);
    ret.add(1, crypted);
    return ret;
}

/**
 * clean
 */
protected static void cleanAll() {
    store = null;
    storeName = null;
    key = null;
    alias = null;
    keyPass = null;
    file = null;
    cert = null;
}
}
