/**
 * $RCSfile$
 * version $Revision$
 * created 30.09.2008 10:37:34 by kunina
 * last modified $Date$ by $Author$
 * (C) ООО Крипто-Про 2004-2008.
 *
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован 
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ComLine;

import JTLS_samples.Client;
import JTLS_samples.Server;
import JTLS_samples.connection.SSLConfiguration;
import JTLS_samples.connection.SSLConnector;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.KeyStore.JCPPrivateKeyEntry;
import ru.CryptoPro.JCP.Util.PaneDefaultProvider;
import ru.CryptoPro.JCP.tools.Array;
import ru.CryptoPro.JCP.tools.License;

import ru.CryptoPro.JCPRequest.GostCertificateRequest;
import userSamples.Constants;
import util.ResolveProvider;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.net.ssl.SSLContext;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.*;
import java.security.Provider;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.spec.AlgorithmParameterSpec;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;

/**
 * Проверка работы провайдера.
 *
 * @author Copyright 2004-2008 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class CheckConfFull {

private static boolean f_jcp = false;
private static boolean f_crypto = false;
private static boolean f_jtls = false;
private static boolean f_jcsp = false;
private static Logger log = Logger.getLogger("Log");

private static String[] ALG_NAMES = {
    "GOST R 34.10-2001",
    "GOST R 34.10-2012 (256)",
    "GOST R 34.10-2012 (512)"
};

// Список алгоритмов для генерации контейнера
// на алгоритме подписи
private static String[] kpAlg = {
    JCP.GOST_EL_DEGREE_NAME,
    JCP.GOST_EL_2012_256_NAME,
    JCP.GOST_EL_2012_512_NAME
};

// Список алгоритмов для генерации контейнера
// на алгоритме обмена
private static String[] kpAlgDH = {
    JCP.GOST_EL_DH_NAME,
    JCP.GOST_DH_2012_256_NAME,
    JCP.GOST_DH_2012_512_NAME
};

// Список алгоритмов подписи
private static String[] signAlg = {
    JCP.GOST_EL_SIGN_NAME,
    JCP.GOST_SIGN_2012_256_NAME,
    JCP.GOST_SIGN_2012_512_NAME
};

// список алиасов для подписи
private static String[] signAliases = {
    "jcptestsignkey_2001",
    "jcptestsignkey_2012_256",
    "jcptestsignkey_2012_512"
};

// список алиасов для шифрования
private static String[] clientAliases = {
    "jcptestclientkey_2001",
    "jcptestclientkey_2012_256",
    "jcptestclientkey_2012_512"
};

// список алиасов для шифрования
private static String[] serverAliases = {
    "jcptestserverkey_2001",
    "jcptestserverkey_2012_256",
    "jcptestserverkey_2012_512"
};

// список паролей
private static char[][] passwords = {
    "a1234".toCharArray(),
    "p1234".toCharArray(),
    "q1234".toCharArray()
};

private static char[] trustPass = "jcptest".toCharArray();
private static String dNameCN = "CN=";
private static String dNameE = ",OU=Security,O=CryptoPro,C=RU";
private static int sslPort = 1212;
private static String server_store = "jcptestserverkey.store";
private static String key_store = "jcptestclientkey.store";
private static String in = "jcptestdoc.in";
private static String out = "jcptestdoc.out";
private static String out_a = "jcptestdoc_a.out";
private static String doc;
private static String get_doc;
private static String get_doc_a;
private static String trustSt;
private static String keySt;

public static final String OK = "OK";
public static final String FAIL = "FAILURE";
private static final String FORMAT = "%s - %s\n";

public static final Map<String, String> checkedModules =
    new LinkedHashMap<String, String>();

private static final String JCP_PROVIDER_NAME = "JCP";
private static final String CRYPTO_PROVIDER_NAME = "Crypto";
private static final String JTLS_PROVIDER = "JTLS";
private static final String JCSP_PROVIDER_NAME = "JCSP";

// Класс с описанием статуса проверки
private static class CheckStatus {

    // Успех проверки
    private boolean status;

    // Описание проверки
    private String description;

    /**
     * Конструктор.
     *
     * @param status Успех проверки.
     * @param description Описание проверки.
     */
    public CheckStatus(boolean status, String description) {
        this.status = status;
        this.description = description;
    }

    /**
     * Результат проверки.
     *
     * @return результат.
     */
    public boolean getStatus() {
        return status;
    }

    /**
     * Описание проверки.
     *
     * @return описание.
     */
    public String getDescription() {
        return description;
    }

}

static {
    // Включаем возможность онлайновой проверки цепочки.
    System.setProperty("com.sun.security.enableCRLDP", "true");
    System.setProperty("com.ibm.security.enableCRLDP", "true");
}

/**
 * Конструктор.
 *
 */
private CheckConfFull() {
    ;
}

/**
 * Получение строки состояния по результату проверки.
 *
 * @param status Состояние проверки.
 * @return строка состояния.
 */
private static String getStatusInfo(CheckStatus status) {

    StringBuilder result = new StringBuilder();
    result.append(status.getStatus() ? OK : FAIL);
    result.append(" (").append(status.getDescription()).append(")");

    return result.toString();
}

/**
 * Класс для булирования логов в файл.
 *
 */
static class LocalFileHandler extends StreamHandler {

    /**
     * Директория пользователя.
     */
    private static final String USER_DIR = System.getProperty("user.dir");

    /**
     * Файл лога.
     */
    private static final String LOG_FILE = USER_DIR + File.separator + "CheckConfFull.log";

    /**
     * Конструктор.
     *
     * @throws Exception
     */
    public LocalFileHandler() throws Exception {
        super(new FileOutputStream(LOG_FILE), new SimpleFormatter());
    }

}

/**
 * CheckConfFull [-servDir C:/*.*]
 * <p/>
 * <DL> <DT><b> -servDir </b>  <DD>рабочая директория<DD>(по умолчанию
 * текущая)</DD></DT> </DL>
 *
 * @param args аргументы командной строки
 * @throws IOException /
 */
public static void main(String[] args) throws Exception {

    log.addHandler(new LocalFileHandler());

    CheckStatus status;
    boolean isJtlsServer = true;

    if (ComLine.getFunc(ComLine.hllp, args)) {
        log.info(ComLine.CheckConfFullHelp);
    }
    else {

        checkProviders();

        // server working dir
        final String dir = ComLine.getValue(ComLine.servDir,
            args, new File(".").getCanonicalPath());

        if (f_jtls) {
            ru.CryptoPro.ssl.ServerLicense serverLicense = new ru.CryptoPro.ssl.ServerLicense();
            isJtlsServer = serverLicense.isServer();
        } // if

        if (f_jcp) {

            gen(JCP.HD_STORE_NAME, JCP.PROVIDER_NAME, JCP.PROVIDER_NAME,
                kpAlg, signAlg, signAliases, false, false);

        } // if

        if (f_crypto || f_jtls) {

            gen(JCP.HD_STORE_NAME, JCP.PROVIDER_NAME, CRYPTO_PROVIDER_NAME,
                kpAlgDH, signAlg, clientAliases, false, false);

            gen(JCP.HD_STORE_NAME, JCP.PROVIDER_NAME, CRYPTO_PROVIDER_NAME,
                kpAlgDH, signAlg, serverAliases, true, false);

        } // if

        if (f_jtls) {
                prepareTls(dir, JCP.HD_STORE_NAME, JCP.PROVIDER_NAME);
        } // if

        if (f_jcp) {

            // Если провайдер по умолчанию - Java CSP,
            // то перед тестами JCP первым ставим JCP.

            if (PaneDefaultProvider.getDefaultProviderIndexCached(null) ==
                PaneDefaultProvider.DEFAULT_PROVIDER_CSP) {

                ResolveProvider.insertProvider(JCP.PROVIDER_NAME,
                    JCP.class.getName(), 1);

            } // if

            // 1. Проверка подписи.

            status = checkSignature(JCP.HD_STORE_NAME, JCP.PROVIDER_NAME);
            checkedModules.put(JCP.PROVIDER_NAME, getStatusInfo(status));
            logResult(JCP.PROVIDER_NAME, status.getStatus(), getStatusInfo(status));

            if (f_crypto) {

                // Если провайдер по умолчанию - Java CSP,
                // то после JCP ставим вторым JCryptoP.

                if (PaneDefaultProvider.getDefaultProviderIndexCached(null) ==
                    PaneDefaultProvider.DEFAULT_PROVIDER_CSP) {

                    ResolveProvider.insertProvider(
                        ru.CryptoPro.Crypto.CryptoProvider.PROVIDER_NAME,
                        ru.CryptoPro.Crypto.CryptoProvider.class.getName(),
                        1);

                } // if

                // 2. Проверка шифрования.

                status = checkEncDec(JCP.HD_STORE_NAME, JCP.PROVIDER_NAME,
                    CRYPTO_PROVIDER_NAME);

                checkedModules.put(CRYPTO_PROVIDER_NAME, getStatusInfo(status));
                logResult(CRYPTO_PROVIDER_NAME, status.getStatus(), getStatusInfo(status));

                if (f_jtls) {

                    // 3. Проверка TLS. Выполняем TLS-проверки только
                    // в том случае, если гарантировано поддерживается
                    // создание серверного сокета, т.е. лицензия Java
                    // TLS - серверная.

                    if (isJtlsServer) {

                        status = checkTLS(false, dir, JCP.HD_STORE_NAME,
                            JCP.PROVIDER_NAME, CRYPTO_PROVIDER_NAME,
                            JCP.CERT_STORE_NAME, JCP.PROVIDER_NAME,
                            false);

                        checkedModules.put(JTLS_PROVIDER, getStatusInfo(status));
                        logResult(JTLS_PROVIDER, status.getStatus(), getStatusInfo(status));

                    } // if
                    else {

                        final String reason = "SKIPPED (reason: Java TLS has a client" +
                            " license, but a server license is required)";

                        checkedModules.put(JTLS_PROVIDER, reason);
                        logResult(JTLS_PROVIDER, false, reason);

                    } // else

                } // if java tls

            } // if crypto

        } // if jcp

        if (f_jcsp) {

            // Если провайдер по умолчанию - JCP,
            // то перед тестами Java CSP первым
            // ставим Java CSP.

            ResolveProvider.insertJavaCSPProvider();

            boolean gost2012disabled = false;
            if (Arrays.asList(args).contains("-gost2012disabled")) {
                gost2012disabled = true;
            } // if

            gen(ComLine.JCSP_DEFAULT_STORE_TYPE, JCSP_PROVIDER_NAME,
                JCSP_PROVIDER_NAME, kpAlg, signAlg, signAliases, false,
                gost2012disabled);

            gen(ComLine.JCSP_DEFAULT_STORE_TYPE, JCSP_PROVIDER_NAME,
                JCSP_PROVIDER_NAME, kpAlgDH, signAlg, clientAliases, false,
                gost2012disabled);

            gen(ComLine.JCSP_DEFAULT_STORE_TYPE, JCSP_PROVIDER_NAME,
                JCSP_PROVIDER_NAME, kpAlgDH, signAlg, serverAliases, true,
                gost2012disabled);

            if (f_jtls) {
                prepareTls(dir, ComLine.JCSP_DEFAULT_STORE_TYPE,
                    JCSP_PROVIDER_NAME);
            } // if

            // 1. Проверка подписи.

            status = checkSignature(ComLine.JCSP_DEFAULT_STORE_TYPE,
                JCSP_PROVIDER_NAME);

            checkedModules.put(JCSP_PROVIDER_NAME + " (Signature)",
                getStatusInfo(status));

            logResult(JCSP_PROVIDER_NAME + " (Signature)",
                status.getStatus(), getStatusInfo(status));

            // 2. Проверка шифрования.

            status = checkEncDec(ComLine.JCSP_DEFAULT_STORE_TYPE,
                JCSP_PROVIDER_NAME, JCSP_PROVIDER_NAME);

            checkedModules.put(JCSP_PROVIDER_NAME +
                " (Encryption/Decryption)", getStatusInfo(status));

            logResult(JCSP_PROVIDER_NAME + " (Encryption/Decryption)",
                status.getStatus(), getStatusInfo(status));

            if (f_jtls) {

                // 3. Проверка TLS. Выполняем TLS-проверки только
                // в том случае, если гарантировано поддерживается
                // создание серверного сокета, т.е. лицензия Java
                // TLS - серверная.

                if (isJtlsServer) {

                    ru.CryptoPro.JCSP.JCSPLicense jCspLicense = new ru.CryptoPro.JCSP.JCSPLicense();
                    int cspLicenseType = jCspLicense.getCSPLicenseType();

                    // Выполняем TLS-проверки только в том случае, если
                    // гарантировано поддерживается создание серверного
                    // сокета, т.е. лицензии CSP и Java CSP - серверные.

                    if (cspLicenseType != License.CSP_CLIENT_LICENSE) {

                        status = checkTLS(true, dir, ComLine.JCSP_DEFAULT_STORE_TYPE,
                            JCSP_PROVIDER_NAME, JCSP_PROVIDER_NAME, JCP.CERT_STORE_NAME,
                            JCP.PROVIDER_NAME, gost2012disabled);

                        checkedModules.put(JCSP_PROVIDER_NAME + " (" +
                            JTLS_PROVIDER + ")", getStatusInfo(status));

                        logResult(JCSP_PROVIDER_NAME + " (" + JTLS_PROVIDER + ")",
                            status.getStatus(), getStatusInfo(status));

                    } // if
                    else {

                        final String reason = "SKIPPED (reason: CSP has a client" +
                            " license, but a server license is required)";

                        checkedModules.put(JCSP_PROVIDER_NAME + " (" + JTLS_PROVIDER + ")", reason);
                        logResult(JCSP_PROVIDER_NAME + " (" + JTLS_PROVIDER + ")", false, reason);

                    } // else

                } // if
                else {

                    final String reason = "SKIPPED (reason: Java TLS has a client" +
                        " license, but a server license is required)";

                    checkedModules.put(JCSP_PROVIDER_NAME + " (" + JTLS_PROVIDER + ")", reason);
                    logResult(JCSP_PROVIDER_NAME + " (" + JTLS_PROVIDER + ")", false, reason);

                } // else

            } // if

        } // if java csp

    } // else

    // Вывод результата.

    log.info("\n*******************************\n");
    String result = "\n";

    for (String key : checkedModules.keySet()) {
        result += "\t" + String.format(FORMAT, key, checkedModules.get(key));
    } // for

    log.info(result);

}

/**
 * Генерация ключевой пары.
 *
 * @param storeType Тип контейнера.
 * @param storeProvider Провайдер контейнера.
 * @param genProvider Провайдер ключа.
 * @param algList Список алгоритмов ключей.
 * @param sigAlgList Список алгоритмов подписи.
 * @param aliases Список алиасов.
 * @param isServer Флаг серверного сертификата.
 * @param gost2012disabled Флаг тестирования ГОСТ 2012.
 * @throws Exception
 */
private static void gen(String storeType, String storeProvider, String
    genProvider, String[] algList, String[] sigAlgList, String[] aliases,
    boolean isServer, boolean gost2012disabled) throws Exception {

    int size = gost2012disabled ? 1 : algList.length;
    if (gost2012disabled) {
        log.info("GOST 2012 disabled for TLS tests with Java CSP.");
    } // if

    log.info("Prepare containers");
    for (int i = 0; i < size; i++) {

        String kpAlgorithm = algList[i];
        String signAlgorithm = sigAlgList[i];
        String alias = aliases[i];
        char[] password = passwords[i];

        log.info("*** Check " + genProvider + " key pair generation on " +
            kpAlgorithm + " with signature algorithm " + signAlgorithm +
                " and writing to store");

        final String dName = dNameCN + alias + dNameE;
        if (keyGen(storeType, storeProvider, genProvider, alias,
            kpAlgorithm, signAlgorithm, password, dName, isServer)) {

            log.info("*** Check " + genProvider + " key pair generation on " +
                kpAlgorithm + " with signature algorithm " + signAlgorithm +
                    " and writing to store - OK");

        } // if

    } // for

    log.info("Containers created");

}

/**
 * Подготовка хранилищ доверенных сертификатов для TLS.
 *
 * @param dir Папка сервера.
 * @param storeType Тип контейнера.
 * @param provider Провайдер.
 * @throws Exception
 */
private static void prepareTls(String dir, String storeType,
    String provider) throws Exception {

    log.info("*** Prepare files");

    doc = dir + File.separator + in;
    get_doc = dir + File.separator + out;
    get_doc_a = dir + File.separator + out_a;

    Array.writeFile(doc, "12345".getBytes());

    if (!new File(doc).exists()) {
        throw new Exception("Can't create file for test");
    } // if

    log.info("*** Prepare TrustStore");

    keySt = dir + File.separator + key_store;
    createTrustStore(storeType, provider, JCP.CERT_STORE_NAME,
        JCP.PROVIDER_NAME, keySt, clientAliases);

    trustSt = dir + File.separator + server_store;
    createTrustStore(storeType, provider, JCP.CERT_STORE_NAME,
        JCP.PROVIDER_NAME, trustSt, serverAliases);

    log.info("*** TrustStore created");

}


/**
 * Запись в лог сообщения о проверке.
 *
 * @param module Имя модуля для вывода в лог.
 * @param status Статус проверки.
 */
private static void logResult(String module, Boolean status, String info) {
    if (status) {
        log.info(String.format("\n" + FORMAT, module, info));
    } else {
        log.warning(String.format("\n" + FORMAT, module, info));
    }
}

/**
 * Проверка установки провайдеров.
 *
 */
private static void checkProviders() {

    final Provider[] providers = Security.getProviders();

    for (int i = 0; i < providers.length; i++) {

        if (JCP_PROVIDER_NAME.equals(providers[i].getName()))
            f_jcp = true;
        if (CRYPTO_PROVIDER_NAME.equals(providers[i].getName()))
            f_crypto = true;
        if (JTLS_PROVIDER.equals(providers[i].getName()))
            f_jtls = true;
        if (JCSP_PROVIDER_NAME.equals(providers[i].getName()))
            f_jcsp = true;

    } // for

    if (!f_jcp)    log.info("Provider " + JCP_PROVIDER_NAME + " not installed");
    if (!f_crypto) log.info("Provider " + CRYPTO_PROVIDER_NAME + " not installed");
    if (!f_jtls)   log.info("Provider " + JTLS_PROVIDER + " not installed");
    if (!f_jcsp)   log.info("Provider " + JCSP_PROVIDER_NAME + " not installed");
}

/**
 * Удаление контейнера.
 *
 * @param storeType Тип ключевого контейнера.
 * @param provider Провайдер для работы с контейнером.
 * @param alias Алиас удаляемого ключа (контейнера).
 */
private static void deleteContainer(String storeType,
    String provider, String alias) {

    log.info("### Delete container " + alias + " by using " +
        " store provider " + provider + " ###");

    try {

        KeyStore keyStore = KeyStore.getInstance(storeType, provider);
        keyStore.load(null, null);
        keyStore.deleteEntry(alias);

        log.info("Deleting of " + alias + " completed");

    } catch (KeyStoreException e) {
        log.warning("Couldn't delete container: " + alias +
            "(" + e.getMessage() + ")");
    } catch (NoSuchProviderException e) {
        log.warning("Couldn't delete container: " + alias +
            "(" + e.getMessage() + ")");
    } catch (CertificateException e) {
        log.warning("Couldn't delete container: " + alias +
            "(" + e.getMessage() + ")");
    } catch (NoSuchAlgorithmException e) {
        log.warning("Couldn't delete container: " + alias +
            "(" + e.getMessage() + ")");
    } catch (IOException e) {
        log.warning("Couldn't delete container: " + alias +
            "(" + e.getMessage() + ")");
    }

}

 /**
 * Проверка создания подписи.
 *
 * @param storeType Тип ключевого контейнера.
 * @param provider Провайдер для подписи и работы с
 * контейнером.
 * @return True, если проверка выполнена успешно.
 */
private static CheckStatus checkSignature(String storeType,
    String provider) {

    log.info("### Check singing and verifying by using " +
        "provider " + provider + " ###");

    boolean ok = true;
    boolean oneCheck = true;
    StringBuilder description = new StringBuilder();

    for (int i = 0; i < kpAlg.length; i++) {

        String signAlgorithm = signAlg[i];
        String signKey = signAliases[i];
        char[] password = passwords[i];

        try {

            log.info("*** Check " + provider + " signature and " +
                "verification with signature algorithm " + signAlgorithm);

            if (signVer(storeType, provider, signAlgorithm, signKey, password)) {

                log.info("*** Check " + provider + " with signature algorithm " +
                    signAlgorithm + " generation|verification - OK");

            } // if
            else {

                log.info("*** Check " + provider + " with signature algorithm " +
                    signAlgorithm + " generation|verification - FAILURE");

            } // else

        } catch (Exception e) {

            ok = false;
            oneCheck = false;

            e.printStackTrace();
            log.log(Level.SEVERE, "Exception", e);

        }

        deleteContainer(storeType, provider, signKey); // удаляем

        description.append(ALG_NAMES[i]);
        description.append(" - ");
        description.append((oneCheck ? OK : FAIL));

        if (i < kpAlg.length - 1) {
            description.append(", ");
        } // if

        oneCheck = true;

    } // for

    return new CheckStatus(ok, description.toString());

}

 /**
 * Проверка возможности шифрования.
 *
 * @param storeType Тип ключевого контейнера.
 * @param storeProvider Провайдер для работы с контейнером.
 * @param edProvider Провайдер для шифрования (генерации
 * ключевой пары).
 * @return True, если проверка выполнена успешно.
 */
private static CheckStatus checkEncDec(String storeType,
    String storeProvider, String edProvider) {

    log.info("### Check encryption and decryption by using " +
        "provider " + edProvider + " ###");

    boolean ok = true;
    boolean oneCheck = true;
    StringBuilder description = new StringBuilder();

    for (int i = 0; i < kpAlgDH.length; i++) {

        String kpAlgorithmDH = kpAlgDH[i];
        String alisaDhKey = clientAliases[i];
        String bobDhKey = serverAliases[i];
        char[] password = passwords[i];

        try {

            log.info("*** Check " + edProvider + " encryption " +
                "with key algorithm " + kpAlgorithmDH);

            if (encryptDecrypt(storeType, storeProvider, storeProvider,
                edProvider, alisaDhKey, bobDhKey, password)) {

                log.info("*** Check " + edProvider +
                    " encryption and decryption - OK");

            } // if

        } catch (Exception e) {

            ok = false;
            oneCheck = false;

            e.printStackTrace();
            log.log(Level.SEVERE, "Exception", e);

        }

        if (!f_jtls) {
            deleteContainer(storeType, storeProvider, bobDhKey);
            deleteContainer(storeType, storeProvider, alisaDhKey);
        } // if

        description.append(ALG_NAMES[i]);
        description.append(" - ");
        description.append((oneCheck ? OK : FAIL));

        if (i < kpAlgDH.length - 1) {
            description.append(", ");
        } // if

        oneCheck = true;

    } // for

    return new CheckStatus(ok, description.toString());

}

/**
 * Проверка TLS.
 *
 * @param JCSPEnabled Флаг использования Java CSP.
 * @param dir Папка сервера.
 * @param keyStoreType Тип ключевого контейнера.
 * @param keyStoreProvider Провайдер для работы с контейнером.
 * @param keyGenProvider Провайдер для генерации ключевой пары.
 * @param trustStoreType тип хранилища доверенных сертификатов.
 * @param trustStoreProvider Провайдер для работы с хранилищем
 * доверенных сертификатов.
 * @param gost2012disabled Флаг использования ГОСТ 2012.
 * @return True, если проверка выполнена успешно.
 */
private static CheckStatus checkTLS(boolean JCSPEnabled, String dir,
    String keyStoreType, String keyStoreProvider, String keyGenProvider,
    String trustStoreType, String trustStoreProvider, boolean gost2012disabled) {

    log.info("### Check TLS by using key provider " + keyGenProvider +
        " and store provider " + keyStoreProvider + " ###");

    boolean ok = true; // полная проверка
    StringBuilder description = new StringBuilder();

    int size = gost2012disabled ? 1 : kpAlgDH.length;
    if (gost2012disabled) {
        log.info("GOST 2012 disabled for TLS tests with Java CSP.");
    } // if

    for (int i = 0; i < size; i++) {

        String kpAlgorithmDH = kpAlgDH[i];
        String signAlgorithm = signAlg[i];
        boolean onePairCheck = false; // результат проверки пары
        String serverKey = serverAliases[i];
        String clientKey = clientAliases[i];
        char[] password = passwords[i];

        log.info("*** Check TLS with key algorithm " + kpAlgorithmDH +
            " and signature algorithm " + signAlgorithm);

        try {

            log.info("*** Test ssl without authentication of client");
            boolean no_auth = tls(JCSPEnabled, keyStoreType, keyStoreProvider,
                trustStoreType, false, dir, get_doc, serverKey, clientKey, password);

            if (no_auth) {
                log.info("*** " + keyStoreProvider + " and " + keyGenProvider +
                    " tls check ssl without authentication of client - OK");
            } // if

            onePairCheck = no_auth;

            log.info("*** Test ssl with authentication of client");
            boolean client_auth = tls(JCSPEnabled, keyStoreType, keyStoreProvider,
                trustStoreType, true, dir, get_doc_a, serverKey, clientKey, password);

            if (client_auth) {
                log.info("*** " + keyStoreProvider + " and " + keyGenProvider +
                    " tls check ssl with authentication of client - OK");
            } // if

            onePairCheck &= client_auth;

        } catch (Exception e) {

            e.printStackTrace();
            log.log(Level.SEVERE, "Exception", e);

        }

        ok &= onePairCheck;

        deleteContainer(keyStoreType, keyStoreProvider, serverKey);
        deleteContainer(keyStoreType, keyStoreProvider, clientKey);

        description.append(ALG_NAMES[i]);
        description.append(" - ");
        description.append((onePairCheck ? OK : FAIL));

        if (i < kpAlgDH.length - 1) {
            description.append(", ");
        } // if

    } // for

    return new CheckStatus(ok, description.toString());

}

/**
 * Создание хранилища доверенных сертификатов.
 *
 * @param keyStoreType Тип ключевого контейнера.
 * @param keyStoreProvider Провайдер для работы с контейнером.
 * @param trustStoreType тип хранилища доверенных сертификатов.
 * @param trustStoreProvider Провайдер для работы с хранилищем
 * доверенных сертификатов.
 * @param stPath Путь к будущему хранилищу.
 * @param aliases Список алиасов сертификатов для получения из
 * ключевого контейнера.
 */
private static void createTrustStore(String keyStoreType,
    String keyStoreProvider, String trustStoreType, String
    trustStoreProvider, String stPath, String[] aliases)
    throws Exception {

    log.info("### Create trusted store by using store provider " +
        trustStoreProvider);

    // Загрузка сертификата из контейнера.

    log.info("Loading of a certificate with type " + keyStoreType +
        " by using provider " + keyStoreProvider);

    final KeyStore ks = KeyStore.getInstance(keyStoreType, keyStoreProvider);
    ks.load(null, null);

    final KeyStore ts = KeyStore.getInstance(trustStoreType, trustStoreProvider);
    ts.load(null, null);

    for (String alias : aliases) {

        final Certificate cert = ks.getCertificate(alias);
        if (cert == null) {
            throw new Exception("Certificate named \"" + alias + "\" not found");
        } // if

        log.info("Loading of a certificate completed");

        // Создание хранилища сертификатов.

        log.info("Creating of a certificate store with type " +
            trustStoreType + " by using provider " + trustStoreProvider);

        ts.setCertificateEntry(alias, cert);

    }

    ts.store(new FileOutputStream(stPath), trustPass);
    log.info("Certificate was moved to store");

}

/**
 * Выполнение TLS соединения.
 *
 * @param keyStoreType Тип ключевого контейнера.
 * @param provider Провайдер для работы с контейнером.
 * @param trustStoreType тип хранилища доверенных сертификатов.
 * @param isAuth True, если требуется аутентификация клиента.
 * @param dir Папка сервера для передачи файлов.
 * @param outPath Папка клиента для сохранения файла.
 * @param clientAlias Алиас клиента.
 * @param serverAlias Алиас сервера.
 * @param password Пароль к ключам.
 * @return True, если соединение выполнено успешно.
 */
private static boolean tls(boolean JCSPEnabled, String keyStoreType,
    String provider, String trustStoreType, boolean isAuth, String dir,
    String outPath, String serverAlias, String clientAlias, char[]
    password) throws Exception {

    log.info("*** Create TLS connection by using provider " + provider +
        " (client authentication - " + isAuth + ")");

    boolean ok = false;
    Server server = null;

    try {

        // Задаем провайдер Java CSP в качестве провайдера по
        // умолчанию для Java TLS, если необходимо.

        log.info("ENABLE " + (JCSPEnabled ? "Java CSP" : "JCP") + " for TLS");
        if (JCSPEnabled) {
            ru.CryptoPro.ssl.util.cpSSLConfig.setDefaultSSLProvider(JCSP_PROVIDER_NAME);
        } // if
        else {
            // Если в панели JCP на вкладке "Алгоритмы" быудет стоять
            // провайдер Java CSP как провйдер по умолчанию, то если
            // default SSL provider не задан (null), может быть взят
            // провайдер из панели, т.к. не задали ни с помощью
            // setDefaultSSLProvider, ни System.setProperty.
            ru.CryptoPro.ssl.util.cpSSLConfig.setDefaultSSLProvider(JCP.PROVIDER_NAME);
        } // else

        // Настройки сервера: параметры соединения.

        log.info("Prepare server connection");

        SSLConfiguration serverSslConfig = new SSLConfiguration(
            JCSPEnabled, trustStoreType, keySt, trustPass,
            isAuth, keyStoreType, serverAlias, password);

        server = new Server(serverSslConfig, sslPort, dir, ComLine.GOST_TLS);
        server.setTimeout(10000000);
        server.start();

        // Настройки клиента: параметры соединения и SSL-контекст.

        log.info("Prepare client connection");

        SSLConfiguration clientSslConfig = new SSLConfiguration(
            JCSPEnabled, trustStoreType, trustSt, trustPass,
            isAuth, keyStoreType, clientAlias, password);

        SSLConnector clientSslConn = new SSLConnector(clientSslConfig);
        clientSslConn.prepare(false);

        SSLContext clientSslContext = clientSslConn.create(ComLine.GOST_TLS);

        final Client client = new Client("localhost", sslPort);
        client.setTimeout(10000000);

        if (!server.isAlive()) {
            throw new Exception("server not running");
        } // if
        else {
            log.info("server started");
        } // else

        log.info("Execute client request");

        if (client.get(clientSslContext, in, false, outPath, dir) != 0) {
            throw new IOException("Couldn't get data.");
        } // if

        if (!server.isAlive()) {
            throw new Exception("server not alive after client Get");
        } // if
        else {
            log.info("GET complete");
        } // else

        ok = true;

    } catch (Exception e) {

        e.printStackTrace();
        log.log(Level.SEVERE, "Exception", e);

    } finally {

        if (server == null) {
            throw new Exception("server not running at the test");
        } // if

        server.stop();
        Thread.sleep(100);
    }

    return ok;

}

/**
 * Генерирование ключевой пары и запись в хранилище.
 *
 * @param storeType Тип ключевого контейнера.
 * @param storeProvider Провайдер для работы с контейнером.
 * @param provider Провайдер для генерации пары.
 * @param alias Алиас закрытого ключа для сохранения в
 * контейнере.
 * @param keyPairAlgorithm Алгоритм генерации пары.
 * @param signatureAlgorithm Алгоритм подписи.
 * @param keyPass Пароль для сохранения закрытого ключа.
 * @param name DN-имя в для генерации сертификата.
 * @param isServer True, если генерируется серверный ключ.
 * @return True, если ключи сгенерированы и сохранены.
 */
private static boolean keyGen(String storeType, String
    storeProvider, String provider, String alias, String
    keyPairAlgorithm, String signatureAlgorithm, char[]
    keyPass, String name, boolean isServer) throws Exception {

    KeyStore keyStore = KeyStore.getInstance(storeType, storeProvider);
    keyStore.load(null, null);

    if (keyStore.containsAlias(alias)) {

        log.info("*** Container " + alias + " exists. Continue");
        return true;

    } // if

    log.info("*** Generate key pair using algorithm " +
        keyPairAlgorithm + " of provider " + provider);

    // Генерирование ключевой пары.

    final KeyPairGenerator kg =
        KeyPairGenerator.getInstance(keyPairAlgorithm, provider);
    final KeyPair pair = kg.generateKeyPair();

    log.info("Generation of key pair completed");

    // Запрос на сертификат.

    final GostCertificateRequest req = new GostCertificateRequest(storeProvider);
    req.init(keyPairAlgorithm, isServer);
    final byte[] encodedCert = req.getEncodedSelfCert(pair, name, signatureAlgorithm);

    log.info("Certificate request completed");

    // Генерирование самоподписанного сертификата.

    final CertificateFactory cf = CertificateFactory.getInstance("X509");
    final Certificate[] certs = new Certificate[1];
    certs[0] = cf.generateCertificate(new ByteArrayInputStream(encodedCert));

    log.info("Generation of " + (isServer ? "server" : "client") +
        " certificate completed");

    // Запись в хранилище ключевой пары с самоподписанным
    // сертификатом.

    final PrivateKey key = pair.getPrivate();

    if (provider.equalsIgnoreCase(JCSP_PROVIDER_NAME)) {

        log.info("Save key pair by using " + JCSP_PROVIDER_NAME);

        final KeyStore.ProtectionParameter protectedParam =
            new KeyStore.PasswordProtection(keyPass);

        final KeyStore.Entry entry = new JCPPrivateKeyEntry(key, certs);
        keyStore.setEntry(alias, entry, protectedParam);

    } // if
    else {
        keyStore.setKeyEntry(alias, key, keyPass, certs);
    } // else

    keyStore.store(null, null);

    log.info("Recording of a private key named \"" + alias + "\" with type " +
        storeType + " by using " + storeProvider + " completed");

    return true;

}

/**
 * Создание и проверка подписи.
 *
 * @param keyStoreType Тип ключевого контейнера.
 * @param provider Провайдер для работы с контейнером
 * и создания/проверки подписи.
 * @param signAlgorithm Алгоритм подписи.
 * @param alias Алиас ключа/сертификата для подписи/проверки.
 * @param key_pass Пароль для доступа к ключу.
 * @return True, если подпись верна, иначе false.
 */
private static boolean signVer(String keyStoreType, String provider,
    String signAlgorithm, String alias, char[] key_pass) throws Exception {

    log.info("*** Sign data and verify signature by using key " +
        alias + " on algorithm " + signAlgorithm + " of provider " +
            provider);

    // Загрузка ключа из хранилища.

    final KeyStore ks = KeyStore.getInstance(keyStoreType, provider);
    ks.load(null, null);

    PrivateKey privateKey = null;
    Certificate cert = null;

    if (provider.equalsIgnoreCase(JCSP_PROVIDER_NAME)) {

        log.info("Save key pair by using " + JCSP_PROVIDER_NAME);

        final KeyStore.ProtectionParameter protectedParam =
            new KeyStore.PasswordProtection(key_pass);
        final JCPPrivateKeyEntry entry = (JCPPrivateKeyEntry)
            ks.getEntry(alias, protectedParam);

        privateKey = entry.getPrivateKey();
        cert = entry.getCertificate();

    } // if
    else {
        privateKey = (PrivateKey) ks.getKey(alias, key_pass);
        cert = ks.getCertificate(alias);
    } // else

    if (privateKey == null) {
        throw new Exception("Key named \"" + alias + "\" not found");
    } // if

    log.info("Loading of a private key completed");

    // Текст.

    final byte[] text = "sample".getBytes();
    log.info("Loading of a text completed");

    // Генерирование ЭЦП.

    final java.security.Signature sig =
        java.security.Signature.getInstance(signAlgorithm, provider);
    sig.initSign(privateKey);
    sig.update(text);

    // Подпись.

    final byte[] signature = sig.sign();
    log.info("Generation of the signature completed");

    // Загрузка открытого ключа из хранилища.

    ks.load(null, null);

    final PublicKey publicKey = cert.getPublicKey();
    if (publicKey == null) {
        throw new Exception("Key named \"" + alias + "\" not found");
    } // if

    log.info("Loading of a public key completed");

    // Проверка подписи.

    final java.security.Signature sign =
        java.security.Signature.getInstance(signAlgorithm, provider);
    sign.initVerify(publicKey);
    sign.update(text);

    final boolean verifies = sign.verify(signature);

    final String s;
    if (verifies) {
        s = "The signature is valid";
    } // if
    else {
        s = "The signature is not invalid";
    } // else

    log.info(s);
    return verifies;
}

/**
 * Зашифрование и расшифрование данных на
 * ключах согласования.
 *
 * @param storeType Тип ключевого контейнера.
 * @param storeProvider Провайдер для работы с контейнером
 * и создания/проверки подписи.
 * @param rndProvider Провайдер для получения случайной
 * последовательности (IV).
 * @param edProvider Провайдер шифрования.
 * @param alisaDhKey Алиас ключа отправителя.
 * @param bobDhKey Алиас ключа получателя.
 * @param password Пароль к ключам.
 * @return True, если шифрование выполнено успешно, иначе
 * false.
 * @throws Exception
 */
private static boolean encryptDecrypt(String storeType,
    String storeProvider, String rndProvider, String
    edProvider, String alisaDhKey, String bobDhKey,
    char[] password) throws Exception {

    // Текст.
    final byte[] SAMPLE_TEXT = "Classic encryption/decryption".getBytes();
    // Длина вектора.
    final int RND_LENGTH = 8;
    // Алгоритм шифрования.
    final String CIPHER_ALG = "GOST28147/CNT/NoPadding";

    log.info("*** Encrypt and decrypt data on algorithm " + CIPHER_ALG +
        " by using provider " + edProvider);

    // Загрузка ключей Алисы из хранилища.

    log.info("Prepare Alisa's container");

    final KeyStore ks = KeyStore.getInstance(storeType, storeProvider);
    ks.load(null, null);

    PrivateKey alisaPrivateKey = null;
    Certificate alisaCert = null;

    if (storeProvider.equalsIgnoreCase(JCSP_PROVIDER_NAME)) {

        log.info("Loading key pair by using " + JCSP_PROVIDER_NAME);

        final KeyStore.ProtectionParameter protectedParam =
            new KeyStore.PasswordProtection(password);
        final JCPPrivateKeyEntry entry = (JCPPrivateKeyEntry)
            ks.getEntry(alisaDhKey, protectedParam);

        alisaPrivateKey = entry.getPrivateKey();
        alisaCert = entry.getCertificate();

    } // if
    else {
        alisaPrivateKey = (PrivateKey) ks.getKey(alisaDhKey, password);
        alisaCert = ks.getCertificate(alisaDhKey);
    } // else

    if (alisaPrivateKey == null) {
        throw new Exception("Key named \"" + alisaDhKey + "\" not found");
    } // if

    if (alisaCert == null) {
        throw new Exception("Certificate named \"" + alisaDhKey +
            "\" not found");
    } // if

    log.info("Loading of Alisa's private key and certificate completed");

    // Загрузка ключей Боба из хранилища.

    log.info("Prepare Bob's container");

    PrivateKey bobPrivateKey = null;
    Certificate bobCert = null;

    if (storeProvider.equalsIgnoreCase(JCSP_PROVIDER_NAME)) {

        log.info("Loading key pair by using " + JCSP_PROVIDER_NAME);

        final KeyStore.ProtectionParameter protectedParam =
            new KeyStore.PasswordProtection(password);
        final JCPPrivateKeyEntry entry = (JCPPrivateKeyEntry)
            ks.getEntry(bobDhKey, protectedParam);

        bobPrivateKey = entry.getPrivateKey();
        bobCert = entry.getCertificate();

    } // if
    else {
        bobPrivateKey = (PrivateKey)ks.getKey(bobDhKey, password);
        bobCert = ks.getCertificate(bobDhKey);
    } // else

    if (bobPrivateKey == null) {
        throw new Exception("Key named \"" + bobDhKey + "\" not found");
    } // if

    if (bobCert == null) {
        throw new Exception("Certificate named \"" + bobDhKey + "\" not found");
    } // if

    log.info("Loading of Bob's private key and certificate completed");

    // Генерирование начальной синхропосылки для
    // выработки ключа согласования.

    final byte[] sv = new byte[RND_LENGTH];
    final SecureRandom random = SecureRandom.getInstance(
        Constants.RANDOM_ALG, rndProvider);
    random.nextBytes(sv);
    final IvParameterSpec ivSpec = new IvParameterSpec(sv);

    log.info("Random sequence is generated");

    // Выработка ключа согласования алисы c SV.

    final KeyAgreement alisaKeyAgree =
        KeyAgreement.getInstance(alisaPrivateKey.getAlgorithm(), edProvider);
    alisaKeyAgree.init(alisaPrivateKey, ivSpec, null);
    alisaKeyAgree.doPhase(bobCert.getPublicKey(), true);
    final SecretKey alisaAgree =
        alisaKeyAgree.generateSecret(Constants.CHIPHER_ALG);

    log.info("Alisa's agreement key completed");

    // Зашифрование текста на ключе согласования алисы.

    Cipher cipher = Cipher.getInstance(CIPHER_ALG, edProvider);
    cipher.init(Cipher.ENCRYPT_MODE, alisaAgree, (AlgorithmParameterSpec)null, null);

    // Передача вектора инициализации бобу.

    final byte[] iv = cipher.getIV();
    final byte[] encryptedText = cipher.doFinal(
        SAMPLE_TEXT, 0, SAMPLE_TEXT.length);

    log.info("Encryption by using of Alisa's key completed");

    // Выработка ключа согласования боба с тем же SV.

    final KeyAgreement bobKeyAgree =
        KeyAgreement.getInstance(bobPrivateKey.getAlgorithm(), edProvider);
    bobKeyAgree.init(bobPrivateKey, ivSpec, null);
    bobKeyAgree.doPhase(alisaCert.getPublicKey(), true);
    final SecretKey bobAgree =
        bobKeyAgree.generateSecret(Constants.CHIPHER_ALG);

    log.info("Bob's agreement key completed");

    // Расшифрование текста на ключе согласования боба.
    // IV передан от алисы.

    cipher = Cipher.getInstance(CIPHER_ALG, edProvider);
    cipher.init(Cipher.DECRYPT_MODE, bobAgree,
        new IvParameterSpec(iv), null);
    final byte[] decryptedText = cipher.doFinal(
        encryptedText, 0, encryptedText.length);

    log.info("Decryption by using of Bob's key completed");

    // Проверка результата.

    if (decryptedText.length != SAMPLE_TEXT.length) {
        throw new Exception("Invalid length of decrypted data");
    } // if

    for (int i = 0; i < decryptedText.length; i++) {
        if (SAMPLE_TEXT[i] != decryptedText[i]) {
            throw new Exception("Invalid value of decrypted data");
        } // if
    } // for

    log.info("Decrypted data is valid");

    return true;
}

}
