/**
 * $RCSfileExamplesActivity.java,v $
 * version $Revision: 36379 $
 * created 01.12.2014 18:41 by Yevgeniy
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 *
 * Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp;

import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.*;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import ru.CryptoPro.ACSPClientApp.util.*;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.HashData;

import ru.CryptoPro.JCSP.support.BKSTrustStore;
import ru.cprocsp.ACSP.tools.common.AppUtils;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Constructor;
import java.util.*;

/**
 *  Вкладка для запуска всех примеров с использованием
 *  контейнеров.
 *
 * @author Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class ExamplesActivity extends Fragment implements
    AdapterView.OnItemSelectedListener, Observer {

    /**
     * Пакет с примерами.
     */
    private static final String EXAMPLE_PACKAGE = "ru.CryptoPro.ACSPClientApp.client.example.";

    /**
     * Флаг, означающий, установлены ли программно корневые
     * сертификаты для примеров CAdES подписи.
     */
    //private boolean cAdESCAInstalled = false;

    /**
     * Список имен классов примеров.
     */
    private String[] exampleClassesToBeExecuted = null;

    /**
     * Адаптер списка алиасов контейнеров.
     */
    private ArrayAdapter<String> containerAliasAdapter = null;

    /**
     * Список примеров, требующих записанный в поле пароль.
     */
    private String[] examplesRequireWrittenPin = null;

    /**
     * Список примеров, требующих контейнер получателя/сервера.
     */
    private String[] examplesRequireServerContainer = null;

    /**
     * Списки примеров и контейнеров.
     */
    private Spinner spExamplesList = null, spClientList = null, spServerList = null;

    /**
     * Поле для ввода пина к контейнеру с ключом подписи или обмена.
     */
    private EditText etClientPin = null;

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        update(null, null);

    }

    @Override
    public View onCreateView(LayoutInflater inflater,
        ViewGroup container, Bundle savedInstanceState) {

        View page = inflater.inflate(R.layout.examples, container, false);

        examplesRequireWrittenPin = getResources().getStringArray(R.array.ExampleRequireWrittenPin);
        examplesRequireServerContainer = getResources().getStringArray(R.array.ExampleRequireServerContainer);
        exampleClassesToBeExecuted = getResources().getStringArray(R.array.ExampleClasses);

        // Список примеров.

        spExamplesList = (Spinner) page.findViewById(R.id.spExamplesList);

        // Создаем ArrayAdapter для использования строкового
        // массива и способа отображения объекта.

        ArrayAdapter<CharSequence> examplesAdapter =
            ArrayAdapter.createFromResource(page.getContext(),
                R.array.ExamplesDescription, android.R.layout.simple_spinner_item);

        // Способ отображения.

        examplesAdapter.setDropDownViewResource(
            android.R.layout.simple_spinner_dropdown_item);

        spExamplesList.setAdapter(examplesAdapter);
        spExamplesList.setOnItemSelectedListener(this);

        // Список клиентских алиасов.

        spClientList = (Spinner) page.findViewById(R.id.spExamplesClientList);

        containerAliasAdapter = new ArrayAdapter<String>(
            page.getContext(), android.R.layout.simple_spinner_item);

        // Способ отображения.

        containerAliasAdapter.setDropDownViewResource(
            android.R.layout.simple_spinner_dropdown_item);

        spClientList.setAdapter(containerAliasAdapter);
        spClientList.setOnItemSelectedListener(this);

        // Список серверных алиасов.

        spServerList = (Spinner) page.findViewById(R.id.spExamplesServerList);
        spServerList.setAdapter(containerAliasAdapter);
        spServerList.setOnItemSelectedListener(this);

        // Поле ввода пинов.

        etClientPin = (EditText) page.findViewById(R.id.etExamplesClientPassword);

        // Кнопка выполнения.

        initExecuteButton(page);

        AppUtils.setupUI(page.findViewById(R.id.llExamplesMain));
        return page;

    }

    /**
     * Установка сертификатов для CAdES путем выполнения примера
     * установки. Необходима, т.к. неизвестно, установил ли клиент
     * сертификаты в Settings->Security (android >= 4.0).
     *
     * Сейчас не используется, т.к. все необходимые для тестов
     * корневые сертификаты находятся в хранилище cacerts,
     * которое берется из ACSP.
     */
    /*
    private void checkCAdESCACertsAndInstall() {

        // Установка корневых сертификатов для CAdES примеров.
        if (!cAdESCAInstalled) {

            String message = String.format(getString(R.string.ExamplesInstallCAdESCAWarning),
                "InstallCAdESTestTrustCertExample");

            ContainerAdapter adapter = new ContainerAdapter(getActivity(), null, false);
            adapter.setProviderType(ProviderType.currentProviderType());
            adapter.setResources(getResources());

            try {

                CAdESData installRootCert = new InstallCAdESTestTrustCertExample(adapter);

                // Если сертификаты не установлены, сообщаем об
                // этом и устанавливаем их.
                if (!installRootCert.isAlreadyInstalled()) {

                    // Предупреждение о выполнении установки.
                    AppUtils.errorMessage(getActivity(), message, false, false);

                    Logger.clear();
                    Logger.log("*** Forced installation of CA certificates (CAdES) ***");

                    // Установка.
                    installRootCert.getResult();

                } // if

                cAdESCAInstalled = true;

            } catch (Exception e) {
                Logger.setStatusFailed();
                Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);
            }

        }

    }
    */

    @Override
    public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {

        ProviderType.currentProviderType();

        switch (adapterView.getId()) {

            case R.id.spExamplesList: {

                // Выбранный пример.
                int selectedExample = adapterView.getSelectedItemPosition();
                String exampleClass = exampleClassesToBeExecuted[selectedExample];

                // Если пример не требует ввода пина, то блокируем поле ввода пина.
                // Большинство примеров вообще не нуждается в указании пароля, т.к.
                // он будет запрошен в специальном окне. Только TLS примеры требуют
                // клиентский пароль.
                List<String> exampleRWPClasses = Arrays.asList(examplesRequireWrittenPin);

                if (!exampleRWPClasses.contains(exampleClass)) {
                    etClientPin.setText("");
                    etClientPin.setEnabled(false);
                } // if
                else {
                    etClientPin.setEnabled(true);
                    etClientPin.setSelected(true);
                    etClientPin.requestFocus();
                } // else

                // Если пример требует указания алиаса и пина получателя, то
                // разблокируем поле ввода алиаса и пина получателя. Например,
                // примеры шифрования нуждаются в них.

                List<String> exampleRSCClasses = Arrays.asList(examplesRequireServerContainer);
                if (!exampleRSCClasses.contains(exampleClass)) {
                    spServerList.setEnabled(false);
                } // if
                else {
                    spServerList.setEnabled(true);
                } // else

            }
            break;

            case R.id.spExamplesClientList: {
                etClientPin.setText(""); // очистка
            }
            break;

        } // switch

    }

    @Override
    public void onNothingSelected(AdapterView<?> adapterView) {
        ;
    }

    /**
     * Выполнение примера.
     *
     * @param i Номер примера.
     */
    private void executeExample(int i) {

        String exampleDescription = (String) spExamplesList.getItemAtPosition(i);
        String exampleClassName = exampleClassesToBeExecuted[i];

        Logger.clear();
        Logger.log("*** " + exampleDescription + " (" + exampleClassName + ") ***");

        try {

            // Поиск примера.

            final String fullExampleClassName = EXAMPLE_PACKAGE + exampleClassName;
            Class exampleClass = Class.forName(fullExampleClassName);
            Constructor exampleConstructor = exampleClass.getConstructor(ContainerAdapter.class);

            // Сборка универсального ContainerAdapter.

            // Клиентский контейнер (подписант, отправитель, TLS).
            String clientAlias = (String) spClientList.getSelectedItem();
            CharSequence clientPasswordSequence = etClientPin.getText();
            char[] clientPassword = null;

            if (clientPasswordSequence != null) {
                clientPassword = clientPasswordSequence.toString().toCharArray();
            } // if

            // Контейнер получателя.
            String serverAlias = (String) spServerList.getSelectedItem();

            // Настройки примера.

            ContainerAdapter adapter = new ContainerAdapter(getActivity(),
                clientAlias, clientPassword, serverAlias, null);

            adapter.setProviderType(ProviderType.currentProviderType());
            adapter.setResources(getResources()); // для примера установки сертификатов

            boolean clientAuth = Arrays.asList(examplesRequireWrittenPin).contains(exampleClassName); // для TLS примеров

            // Используется общее для всех хранилище корневых
            // сертификатов cacerts.

            final String trustStorePath = getActivity().getApplicationInfo().dataDir +
                File.separator + BKSTrustStore.STORAGE_DIRECTORY + File.separator +
                    BKSTrustStore.STORAGE_FILE_TRUST;

            Logger.log("Example trust store: " + trustStorePath);

            adapter.setTrustStoreProvider(BouncyCastleProvider.PROVIDER_NAME);
            adapter.setTrustStoreType(BKSTrustStore.STORAGE_TYPE);

            adapter.setTrustStoreStream(new FileInputStream(trustStorePath));
            adapter.setTrustStorePassword(BKSTrustStore.STORAGE_PASSWORD);

            // Настройки для подключения к удаленному хосту в зависимости
            // от алгоритма (чтобы охватить по возможности все алгоритмы)
            // для TLS примеров, примера построения цепочки и т.п.

            switch (adapter.getProviderType()) {

                case pt2001: {

                    // Для TLS примеров.

                    if (clientAuth) {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2001ClientAuth);
                    } // if
                    else {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2001NoAuth);
                    } // else

                }
                break;

                case pt2012Short: {

                    // Для TLS примеров.

                    if (clientAuth) {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2012256ClientAuth);
                    } // if
                    else {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2012256NoAuth);
                    } // else

                }
                break;

                case pt2012Long: {

                    // Для TLS примеров.

                    if (clientAuth) {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2012512ClientAuth);
                    } // if
                    else {
                        adapter.setConnectionInfo(RemoteConnectionInfo.host2012512NoAuth);
                    } // else

                }
                break;

            } // switch

            // Выполнение примера.

            HashData exampleImpl = (HashData) exampleConstructor.newInstance(adapter);
            exampleImpl.getResult();

        } catch (Exception e) {
            Logger.setStatusFailed();
            Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);
        }

    }

    @Override
    public void update(Observable observable, Object data) {

        if (containerAliasAdapter != null) {

            try {
                update();
            } catch (Exception e) {
                Logger.log("Can not update container list: " + e.getMessage());
                Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);
            }

        } // if

    }

    /**
     * Обновление списка контейнеров.
     *
     * @throws Exception
     */
    private void update() throws Exception {

        if (getActivity() == null) {

            Logger.log("Invalid context!");
            return;

        }

        // Обновляем список контейнеров.

        AsyncTask<Void, Void, Void> updateContainerListTask = new UpdateContainerListTask();
        updateContainerListTask.execute();

    }

    /**
     * Асинхронная задача обновления списка контейнеров.
     *
     */
    class UpdateContainerListTask extends AsyncTask<Void, Void, Void> {

        /**
         * Окно ожидания.
         */
        private final ProgressDialogHolder progressDialog;

        /**
         * Список контейнеров.
         */
        private final List<String> aliasesList = new ArrayList<String>();

        /**
         * Конструктор.
         *
         */
        public UpdateContainerListTask() {
            progressDialog = new ProgressDialogHolder(getActivity(), true);
        }

        @Override
        protected  void onProgressUpdate(Void... values) {
            progressDialog.show();
        }

        @Override
        protected Void doInBackground(Void... params) {

            publishProgress();

            aliasesList.addAll(
                KeyStoreUtil.aliases(
                    KeyStoreType.currentType(),
                    ProviderType.currentProviderType()
            ));

            return null;

        }

        @Override
        protected void onPostExecute(Void result) {

            progressDialog.dismiss();

            containerAliasAdapter.clear();
            containerAliasAdapter.addAll(aliasesList);
            containerAliasAdapter.notifyDataSetChanged();

        }

    }

    /**
     * Добавление обработчика в кнопку выполнения примера.
     *
     */
    private void initExecuteButton(View view) {

        Button btExecuteButton =
            (Button) view.findViewById(R.id.btExamplesExecute);

        // Выполнение примера.
        btExecuteButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                int exampleIndex = spExamplesList.getSelectedItemPosition();
                executeExample(exampleIndex);
            }

        });
    }

}
