/**
 * $RCSfileUtilActivity.java,v $
 * version $Revision: 36379 $
 * created 13.06.2017 16:49 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import android.widget.*;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import ru.CryptoPro.ACSPClientApp.util.*;
import ru.CryptoPro.JCP.tools.Array;

import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.JCSP.support.BKSTrustStore;

import ru.cprocsp.ACSP.tools.common.AppUtils;

import java.io.*;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.*;

/**
 * Вкладка для проверки шифрования сообщений с помощью
 * утилиты CMSUtil (cmsutil.jar). Работа осуществляется
 * только с контейнерами HDIMAGE.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class UtilActivity extends Fragment
    implements AdapterView.OnItemSelectedListener, Observer {

    /**
     * Файл хранилища из ресурсов.
     */
    private static final String CERT_STORE_NAME = "recipient.store";

    /**
     * Пароль к хранилищу сертификатов получателей
     * {@link #CERT_STORE_NAME}.
     */
    private static final char[] CERT_STORE_PASSWORD = "1234".toCharArray();

    /**
     * Интент выбора файла (ACSP).
     */
    public static final String INTENT_FILE_SELECT = "ru.cprocsp.intent.SELECT_DIR";

    /**
     * Идентификатор запроса выбора входящего файла.
     */
    private static final int FILE_SELECT_CODE_IN = 0;

    /**
     * Идентификатор запроса выбора папки для исходящего
     * файла.
     */
    private static final int FILE_SELECT_CODE_OUT = 1;

    /**
     * Тип контейнера.
     */
    private static final String STORE_TYPE = JCSP.HD_STORE_NAME;

    /**
     * Список получателей. Соответствует списку
     * сертификатов в хранилище сертификатов
     * получателей и контейнерам.
     */
    private Spinner spRecipient = null;

    /**
     * Пароль получателя сообщения. Используется при
     * расшифровании для доступа к контейнеру.
     */
    private EditText etRecipientPassword = null;

    /**
     * Поля ввода путей к входящему и исходщему
     * файлам.
     */
    private EditText etInFile = null, etOutFile = null;

    /**
     * Группа выбора действия - зашифрование или
     * расшифрование.
     */
    private RadioGroup rgEncryptDecrypt = null;

    /**
     * Адаптер списка алиасов контейнеров.
     */
    private ArrayAdapter<String> recipientAdapter = null;

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        update(null, null);

    }

    /**
     * Получение папки приложения для записи хранилища
     * сертификатов получателей.
     *
     * @return папка.
     */
    private String getAppDir() {
        return getActivity().getApplicationInfo().dataDir;
    }

    @Override
    public View onCreateView(LayoutInflater inflater,
        ViewGroup container, Bundle savedInstanceState) {

        View page = inflater.inflate(R.layout.util, container, false);

        // Осуществляемое действие.

        rgEncryptDecrypt = (RadioGroup) page.findViewById(R.id.rgEncryptDecrypt);

        // Список алиасов получателей.

        spRecipient = (Spinner) page.findViewById(R.id.spRecipient);

        // Адаптер списка алиасов получателей.

        recipientAdapter = new ArrayAdapter<String>(page.getContext(),
            android.R.layout.simple_spinner_item);

        recipientAdapter.setDropDownViewResource(
            android.R.layout.simple_spinner_dropdown_item);

        spRecipient.setAdapter(recipientAdapter);
        spRecipient.setOnItemSelectedListener(this);

        // Пароль к контейнеру получателя.

        etRecipientPassword = (EditText) page.findViewById(R.id.etPassword);

        // Пути к входящему и исходящему файлам.

        etInFile = (EditText) page.findViewById(R.id.etInFile);
        etOutFile = (EditText) page.findViewById(R.id.etOutFile);

        // Кнопки выбора файлов.

        Button btOpenInFile = (Button) page.findViewById(R.id.btOpenInFile);
        btOpenInFile.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                final Intent intent = new Intent(INTENT_FILE_SELECT);
                intent.putExtra("onlyDirs", false);
                startActivityForResult(intent, FILE_SELECT_CODE_IN);

            }

        });

        Button btOpenOutFile = (Button) page.findViewById(R.id.btOpenOutFile);
        btOpenOutFile.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                final Intent intent = new Intent(INTENT_FILE_SELECT);
                intent.putExtra("onlyDirs", false);
                startActivityForResult(intent, FILE_SELECT_CODE_OUT);

            }

        });

        // Кнопка зашифрования/расшифрования.

        Button btExecute = (Button) page.findViewById(R.id.btExecute);
        btExecute.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                boolean encrypt = (rgEncryptDecrypt.getCheckedRadioButtonId() == R.id.rbEncrypt);
                encryptOrDecrypt(encrypt);
            }

        });

        // Кнопка сравнения файлов.

        Button btCompare = (Button) page.findViewById(R.id.btCompare);
        btCompare.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                compareFiles();
            }

        });

        AppUtils.setupUI(page.findViewById(R.id.llUtilMain));
        return page;

    }

    @Override
    public void update(Observable observable, Object data) {

        // Обновляем список контейнеров.Работа
        // осуществляется только с контейнерами
        // {@link #STORE_TYPE}.

        List<String> aliasesList = KeyStoreUtil.aliases(
            STORE_TYPE,
            ProviderType.currentProviderType()
        );

        if (recipientAdapter != null) {

            recipientAdapter.clear();
            recipientAdapter.addAll(aliasesList);
            recipientAdapter.notifyDataSetChanged();

        } // if

        // Сохранение хранилища с сертификатами получателей
        // в папку приложения.

        saveCertStore(aliasesList);

    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        switch (requestCode) {

            case FILE_SELECT_CODE_IN: {

                if (resultCode == Activity.RESULT_OK && data != null) {

                    final String chosenFilePath = data.getStringExtra("chosenObject");
                    etInFile.setText(chosenFilePath);

                    // Допишем путь к исходящему файлу.

                    boolean encrypt = (rgEncryptDecrypt.getCheckedRadioButtonId() == R.id.rbEncrypt);
                    etOutFile.setText(chosenFilePath + (encrypt ? ".encrypted" : ".decrypted"));

                } // if

            }
            break;

            case FILE_SELECT_CODE_OUT: {

                if (resultCode == Activity.RESULT_OK && data != null) {

                    final String chosenPath = data.getStringExtra("chosenObject");
                    etOutFile.setText(chosenPath);

                } // if

            }
            break;

        } // switch

        super.onActivityResult(requestCode, resultCode, data);

    }

    /**
     * Сохранение файла хранилища в папке
     * приложения. Работа осуществляется
     * только с контейнерами HDIMAGE.
     *
     */
    private void saveCertStore(List<String> aliases) {

        FileInputStream   storeInput = null;
        FileOutputStream storeOutput = null;

        try {

            String path = getAppDir() + File.separator + CERT_STORE_NAME;
            File file = new File(path);

            KeyStore recipientStore = KeyStore.getInstance(
                BKSTrustStore.STORAGE_TYPE,
                BouncyCastleProvider.PROVIDER_NAME
            );

            if (file.exists()) {
                storeInput = new FileInputStream(file);
                recipientStore.load(storeInput, CERT_STORE_PASSWORD);
            } // if
            else {
                recipientStore.load(null, null);
            } // else

            KeyStore keyStore = KeyStore.getInstance(
                STORE_TYPE,
                JCSP.PROVIDER_NAME
            );

            keyStore.load(null, null);
            boolean saved = false;

            // Сохраняем сертификаты в хранилище,
            // добавляя новые или обновляя старые.

            for (String alias : aliases) {

                X509Certificate cert = (X509Certificate)
                    keyStore.getCertificate(alias);

                if (cert != null) {

                    recipientStore.setCertificateEntry(alias, cert);
                    saved = true;

                } // if

            } // for

            // Если была запись - сохраняем.

            if (saved) {

                storeOutput = new FileOutputStream(file);
                recipientStore.store(storeOutput, CERT_STORE_PASSWORD);

            } // if

        } catch (Exception e) {

            Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);
            Logger.log(e.getMessage());

        } finally {

            if (storeInput != null) {

                try {
                    storeInput.close();
                } catch (IOException e) {
                    // ignore
                }

            } // if

            if (storeOutput != null) {

                try {
                    storeOutput.close();
                } catch (IOException e) {
                    // ignore
                }

            } // if

        }

    }

    /**
     * Выполнение зашифрования или расшифрования.
     *
     * @param encrypt True, если выполняется зашифрование.
     */
    private void encryptOrDecrypt(boolean encrypt) {

        Logger.clear();
        Logger.log("*** CMSUtil (cmsutil.jar, " +
            (encrypt ? "encrypt" : "decrypt") + " ***");

        String certStorePath = getAppDir() + File.separator + CERT_STORE_NAME;
        String alias    = (String) spRecipient.getSelectedItem();
        String password = String.valueOf(etRecipientPassword.getText());

        String inFile  = String.valueOf(etInFile.getText());
        String outFile = String.valueOf(etOutFile.getText());

        String[] args =
            encrypt
            ? new String[] {
                "-encrypt",
                "-certstore",
                certStorePath,
                "-certstoretype",
                BKSTrustStore.STORAGE_TYPE,
                "-certstoreprovider",
                BouncyCastleProvider.PROVIDER_NAME,
                "-pass",
                String.copyValueOf(CERT_STORE_PASSWORD),
                "-alias",
                alias,
                "-in",
                inFile,
                "-out",
                outFile,
                "-provider",
                JCSP.PROVIDER_NAME
            }
            : new String[] {
                "-decrypt",
                "-keystore",
                STORE_TYPE,
                "-pass",
                password,
                "-alias",
                alias,
                "-in",
                inFile,
                "-out",
                outFile,
                "-provider",
                JCSP.PROVIDER_NAME
            };

        for (String arg : args) {
            Logger.log("* " + arg);
        } // for

        try {

            cmsutil.CMSMain.main(args);

            Logger.setStatusOK();
            Logger.log("Completed.");

        } catch (Exception e) {

            Logger.setStatusFailed();
            Logger.log(e.getMessage());

            Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);

        }

    }

    /**
     * Сравнение файлов по содержимому.
     *
     */
    private void compareFiles() {

        Logger.clear();
        Logger.log("*** CMSUtil, compare files ***");

        String inFile  = String.valueOf(etInFile.getText());
        String outFile = String.valueOf(etOutFile.getText());

        Logger.log("* In file: " + inFile);
        Logger.log("* Out file: " + outFile);

        try {

            byte[] inFileContent  = Array.readFile(inFile);
            byte[] outFileContent = Array.readFile(outFile);

            boolean equal = Array.compare(inFileContent, outFileContent);

            Logger.setStatusOK();
            Logger.log("Files are equal: " + equal);

        } catch (Exception e) {

            Logger.setStatusFailed();
            Logger.log(e.getMessage());

            Log.e(Constants.APP_LOGGER_TAG, e.getMessage(), e);

        }

    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        ;
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {
        ;
    }

}
