/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.GenKeyPairData;
import ru.CryptoPro.ACSPClientApp.client.example.base.SignData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCP.KeyStore.JCPPrivateKeyEntry;
import ru.CryptoPro.JCSP.JCSP;

import java.security.KeyStore;
import java.security.cert.Certificate;

/**
 * Класс ChangePasswordExample реализует пример смены
 * пароля на созданный контейнер путем копирования его
 * ключа и сертификата в новый контейнер с тем же именем,
 * но новым паролем, задаваемым программно, без ввода в
 * окне CSP.
 *
 * 25/07/2013
 *
 */
public class ChangePasswordExample extends SignData {

    /**
     * Новый пароль на контейнер.
     */
    public static final char[] NEW_PASSWORD =
        "ANewPassword".toCharArray(); // 12345678

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public ChangePasswordExample(ContainerAdapter adapter) {
        super(adapter, false);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new ChangePasswordThread());
    }

    /**
     * Класс ChangePasswordThread реализует смену
     * пароля в отдельном потоке.
     *
     */
    private class ChangePasswordThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            // Создаем ключевой контейнер.

            Logger.log("Create key container.");

            GenKeyPairData genKeyPairData = new GenKeyPairExample(
                containerAdapter, GenKeyPairData.CAType.msCA, null);

            genKeyPairData.getResult();

            // Загружаем ключевой контейнер.
            Logger.log("Load key container.");

            load(true, KeyStoreType.currentType(),
                genKeyPairData.getStoreAlias(), null);

            // Сохраняем ключ так, чтобы не было окон
            // для ввода пин-кода.

            Logger.log("Prepare protected parameters.");

            KeyStore dstKeyStore = KeyStore.getInstance(
                KeyStoreType.currentType(), JCSP.PROVIDER_NAME);

            dstKeyStore.load(null, null);

            KeyStore.ProtectionParameter protectedParamNew =
                new KeyStore.PasswordProtection(NEW_PASSWORD);

            KeyStore.Entry entry = new JCPPrivateKeyEntry(
                getPrivateKey(), new Certificate[] {getCertificate()});

            Logger.log("Change a password of the container: " +
                new String(NEW_PASSWORD));

            dstKeyStore.setEntry(genKeyPairData.getStoreAlias(),
                entry, protectedParamNew);

            dstKeyStore.store(null, null);

            Logger.log("Password was changed successfully.");
            Logger.setStatusOK();

        }
    }

}
