/**
 * $RCSfileEnvelopedCMSExample.java,v $
 * version $Revision: 36379 $
 * created 19.02.2015 13:51 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 *
 * Copyright 2004-2015 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.EncryptDecryptData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.CAdES.EnvelopedSignature;
import ru.CryptoPro.JCP.tools.Array;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

/**
 * Класс EnvelopedCMSExample реализует пример
 * создания и проверки Enveloped CMS подписи.
 *
 * @author Copyright 2004-2015 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class EnvelopedCMSExample extends EncryptDecryptData {

    /**
     * True, если следует использовать key_transport,
     * иначе - key_agreement.
     */
    private boolean isUseTransport = true;

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     * @param useTransport True, если следует использовать key_transport,
     * иначе - key_agreement.
     */
    public EnvelopedCMSExample(ContainerAdapter adapter, boolean useTransport) {
        super(adapter);
        isUseTransport = useTransport;
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new EnvelopedCMSThread());
    }

    /**
     * Класс CheckCertStatusThread реализует формирование
     * и проверку Enveloped CMS сообщения в отдельном потоке.
     *
     */
    private class EnvelopedCMSThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            Logger.log("Load key container to encrypt CMS.");

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            // Загрузка ключа и сертификата получателя.

            load(askPinInDialog, keyStoreType,
                containerAdapter.getServerAlias(),
                containerAdapter.getServerPassword());

            Logger.log(
                "Recipient certificate: " + getCertificate().getSubjectDN() +
                ", public key: " + getCertificate().getPublicKey());

            // Формирование CMS подписи.

            CMSSignExample cmsSign = new CMSSignExample(false, containerAdapter);
            Logger.log("Load key container to sign CMS.");

            // Загрузка параметров подписанта CMS.

            cmsSign.load(askPinInDialog, keyStoreType,
                containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            PrivateKey signerPrivateKey = cmsSign.getPrivateKey();
            X509Certificate signerCertificate = cmsSign.getCertificate();

            Logger.log("Compute attached signature for message '" +
                Constants.MESSAGE + "' :");

            // Формирование совмещенной CMS подписи.

            byte[] signCms = cmsSign.create(
                Constants.MESSAGE.getBytes(),
                false,
                new PrivateKey[]{signerPrivateKey},
                new Certificate[]{signerCertificate},
                false,
                false
            );

            Logger.log("Prepare enveloped signature.");

            // Создание Enveloped CMS.

            EnvelopedSignature envelopedSignature = new EnvelopedSignature();
            if (isUseTransport) {
                Logger.log("Add KeyTransport recipient.");
                envelopedSignature.addKeyTransRecipient(getCertificate());
            } // if
            else {
                Logger.log("Add KeyAgreement recipient.");
                envelopedSignature.addKeyAgreeRecipient(getCertificate());
            } // else

            ByteArrayOutputStream outputSignatureStream = new ByteArrayOutputStream();

            envelopedSignature.open(outputSignatureStream);
            envelopedSignature.update(signCms);

            Logger.log("Produce Enveloped CMS.");

            envelopedSignature.close();
            outputSignatureStream.close();

            // Сформированная Enveloped CMS.

            byte[] envelopedCms = outputSignatureStream.toByteArray();

            Logger.log("--- ENVELOPED SIGNATURE BEGIN ---");
            Logger.log(envelopedCms, true);
            Logger.log("--- ENVELOPED SIGNATURE END ---");
            Logger.log("Decrypt produced Enveloped CMS.");

            // Расшифрование CMS подписи.

            EnvelopedSignature decryptedSignature = new EnvelopedSignature(
                new ByteArrayInputStream(envelopedCms));

            ByteArrayOutputStream outputDecryptedStream = new ByteArrayOutputStream();

            decryptedSignature.decrypt(getCertificate(), getPrivateKey(),
                outputDecryptedStream);

            outputDecryptedStream.close();
            Logger.log("Check output data of decrypted Enveloped CMS.");

            byte[] decryptedCms = outputDecryptedStream.toByteArray();

            if (!Array.compare(signCms, decryptedCms)) {
                throw new Exception("Invalid decrypted content");
            } // if

            Logger.log("Check decrypted CMS.");

            // Проверяем расшифрованную CMS подпись.

            cmsSign.verify(decryptedCms, new Certificate[]
                {cmsSign.getCertificate()}, null);

            Logger.setStatusOK();

        }

    }


}
