/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.SignData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

/**
 * Класс PrintCertificateContentExample реализует пример
 * вывода содержимого сертификата в лог.
 *
 * 25/07/2013
 *
 */
public class PrintCertificateContentExample extends SignData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public PrintCertificateContentExample(ContainerAdapter adapter) {
        super(adapter, false);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new PrintCertificateContentThread());
    }

    /**
     * Класс PrintCertificateContentThread реализует
     * вывод сертификата в отдельном потоке.
     *
     */
    private class PrintCertificateContentThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);
            Logger.log("Load source key container.");

            load(true, keyStoreType, containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            if (getCertificate() == null) {
                Logger.log("Source certificate is null.");
                return;
            } // if

            Logger.log("*************************");
            Logger.log(getCertificate().toString());

            Logger.setStatusOK();

        }

    }

}
