/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.SignData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCSP.JCSP;
import java.security.Signature;

/**
 * Класс SignExample реализует пример подписи
 * сообщения.
 *
 * 27/05/2013
 *
 */
public class SignExample extends SignData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public SignExample(ContainerAdapter adapter) {
        super(adapter, false);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new SignThread());
    }

    /**
     * Класс SignThread реализует формирование подписи
     * в отдельном потоке.
     *
     */
    private class SignThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {
            sign();
        }

    }

    /**
     * Формирование подписи.
     *
     * @return подпись или null.
     * @throws Exception
     */
    public byte[] sign() throws Exception {

        Logger.log("Load key container to sign data.");

        // Тип контейнера по умолчанию.

        String keyStoreType = KeyStoreType.currentType();
        Logger.log("Default container type: " + keyStoreType);

        // Загрузка ключа и сертификата.

        load(askPinInDialog, keyStoreType,
            containerAdapter.getClientAlias(),
            containerAdapter.getClientPassword());

        if (getPrivateKey() == null) {
            Logger.log("Private key is null.");
            return null;
        } // if

        Logger.log("Init Signature: " +
            algorithmSelector.getSignatureAlgorithmName());

        // Инициализация подписи.

        Signature sn = Signature.getInstance(
            algorithmSelector.getSignatureAlgorithmName(),
            JCSP.PROVIDER_NAME);

        Logger.log("Init signature by private key: " + getPrivateKey());

        sn.initSign(getPrivateKey());
        sn.update(Constants.MESSAGE.getBytes());

        // Формируем подпись.

        Logger.log("Compute signature for message '" +
            Constants.MESSAGE + "' :");

        byte[] sign = sn.sign();

        Logger.log(sign, true);
        Logger.setStatusOK();

        return sign;

    }

}
