/**
 * $RCSfileXMLSignExample.java,v $
 * version $Revision: 36379 $
 * created 18.08.2014 12:02 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 *
 * Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.utils.Constants;
import org.apache.xpath.XPathAPI;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.CryptoPro.ACSPClientApp.client.example.base.XMLData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.AlgorithmSelector;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCPxml.Consts;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;

/**
 * Класс XMLSignVerifyExample реализует пример
 * создания и проверки XML подписи.
 *
 * @author Copyright 2004-2014 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class XMLSignVerifyExample extends XMLData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public XMLSignVerifyExample(ContainerAdapter adapter) {
        super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new XMLSignVerifyThread());
    }

    /**
     * Класс XMLSignVerifyThread реализует подпись
     * и проверку XML в отдельном потоке.
     *
     */
    private class XMLSignVerifyThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            Logger.log("Load key container to sign XML data.");

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            // Загрузка ключа и сертификата.

            load(askPinInDialog, keyStoreType,
                containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            if (getPrivateKey() == null) {
                Logger.log("Private key is null.");
                return;
            } // if

            // Создание документа.

            Logger.log("Creating of XML document.");
            Document doc = createSampleDocument();

            // Вывод в лог.

            byte[] docContent = prepareLogXML(doc);
            Logger.log(docContent, false);

            // Определение алгоритмов.

            String defaultSignAlgUrn = Consts.URN_GOST_SIGN;
            String defaultDigestAlgUrn = Consts.URN_GOST_DIGEST;

            if (providerType.equals(AlgorithmSelector.DefaultProviderType.pt2012Short)) {
                defaultSignAlgUrn = Consts.URN_GOST_SIGN_2012_256;
                defaultDigestAlgUrn = Consts.URN_GOST_DIGEST_2012_256;
            } // if
            else if (providerType.equals(AlgorithmSelector.DefaultProviderType.pt2012Long)) {
                defaultSignAlgUrn = Consts.URN_GOST_SIGN_2012_512;
                defaultDigestAlgUrn = Consts.URN_GOST_DIGEST_2012_512;
            } // else

            // Подпись документа.

            Logger.log("Signing of XML document.");

            signDoc(doc, getPrivateKey(), getCertificate(),
                defaultSignAlgUrn, defaultDigestAlgUrn);

            // Вывод в лог подписанного документа.

            Logger.log("Signed XML document:");

            byte[] docSignedContent = prepareLogXML(doc);
            Logger.log(docSignedContent, false);

            // Проверка подписи документа.

            Logger.log("Verifying of XML document.");

            if (!verifyDoc(doc)) {
                throw new Exception("Invalid XML signature.");
            } // if

            Logger.setStatusOK();

        }

    }

    @Override
    protected Document createSampleDocument()
        throws ParserConfigurationException {

        DocumentBuilderFactory dbf = createDocFactory();
        Document document = dbf.newDocumentBuilder().newDocument();

        Element root = document.createElementNS("http://www.apache.org/ns/#app1",
            "apache:RootElement");
        root.setAttributeNS(Constants.NamespaceSpecNS, "xmlns:apache",
            "http://www.apache.org/ns/#app1");

        document.appendChild(root);
        root.appendChild(document.createTextNode("\n"));

        Element childElement = document.createElementNS(
            "http://www.apache.org/ns/#app1", "apache:foo");
        childElement.appendChild(document.createTextNode("Some simple text"));

        root.appendChild(childElement);
        root.appendChild(document.createTextNode("\n"));

        return document;
    }

    @Override
    protected void signDoc(Document doc, PrivateKey privateKey,
        X509Certificate cert, String signMethod, String digestMethod)
        throws Exception {

        XMLSignature sig = new XMLSignature(doc, "", signMethod);
        Element anElement = doc.getDocumentElement();

        Logger.log("Add signature element.");
        anElement.appendChild(sig.getElement());

        Logger.log("Apply transformations.");
        Transforms transforms = new Transforms(doc);

        transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
        transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);

        sig.addDocument("", transforms, digestMethod);
        sig.addKeyInfo(cert);

        sig.sign(privateKey);
        Logger.log("XML document is signed.");

    }

    /**
     * Извлечение узла подписи.
     *
     * @param doc Подписанный XML документ.
     * @return узел подписи.
     * @throws Exception
     */
    private Element getSignature(Document doc) throws Exception {
        Element context = doc.createElementNS(null, "namespaceContext");
        context.setAttributeNS(Constants.NamespaceSpecNS, "xmlns:" + "ds",
            Constants.SignatureSpecNS);
        return (Element) XPathAPI.selectSingleNode(doc, "//ds:Signature[1]", context);

    }

    @Override
    protected boolean verifyDoc(Document doc) throws Exception {

        Logger.log("Search for signature element.");
        Element sigElement = getSignature(doc);
        XMLSignature signature = new XMLSignature(sigElement, "");

        Logger.log("Search for key information.");
        KeyInfo ki = signature.getKeyInfo();
        X509Certificate certKey = ki.getX509Certificate();

        if (certKey == null) {
            throw new Exception("There are no information about public key.");
        } // if

        Logger.log("Check signature.");
        return signature.checkSignatureValue(certKey);

    }

}
