/**
 * $RCSfileIntegrityExample.java,v $
 * version $Revision: 36379 $
 * created 28.08.2017 17:44 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;
import ru.CryptoPro.JCSP.CSPProviderInterface;

import ru.cprocsp.ACSP.tools.common.CSPIntegrityConstants;
import ru.cprocsp.ACSP.tools.integrity.IntegrityInterface;

import java.util.List;

/**
 * Пример IntegrityExample предназначен для
 * проверки целостности (библиотек провайдера).
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class IntegrityExample implements Executable<Void> {

    /**
     * Параметры провайдера.
     */
    private final CSPProviderInterface providerInfo;

    /**
     * Конструктор.
     *
     * @param providerInfo Параметры провайдера.
     */
    public IntegrityExample(CSPProviderInterface providerInfo) {
        this.providerInfo = providerInfo;
    }

    @Override
    public Void execute() throws Exception {

        IntegrityInterface integrity = providerInfo.getIntegrity();
        Logger.INSTANCE.append("Checking...");

        // Проверка целостности.
        int result = integrity.check(true);

        Logger.INSTANCE.append("Checking completed.");
        Logger.INSTANCE.append("Result: " + result);

        if (result != CSPIntegrityConstants.CHECK_INTEGRITY_SUCCESS) {
            throw new Exception("Integrity corrupted.");
        } // if

        Logger.INSTANCE.append("Last check date: " + integrity.getLastDate());
        int status = integrity.getLastStatus();

        switch (status) {

            case CSPIntegrityConstants.CHECK_INTEGRITY_SUCCESS: {
                Logger.INSTANCE.append("Status: completed");
            }
            break;

            case CSPIntegrityConstants.CHECK_INTEGRITY_INVALID: {
                Logger.INSTANCE.append("Status: failed");
            }
            break;

            default: {
                Logger.INSTANCE.append("Status: unknown");
            }

        } // switch

        // Удобочитаемый список проверяемых библиотек
        // и их хешей.

        List<String> items = integrity
            .getItems(null, null, "");

        for (String item : items) {
            Logger.INSTANCE.append("\t" + item);
        } // for

        return null;

    }

}
