/**
 * $RCSfileSignatureExample.java,v $
 * version $Revision: 36379 $
 * created 21.08.2017 13:12 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCSP.JCSP;

import java.security.PrivateKey;
import java.security.Signature;
import java.security.cert.X509Certificate;

/**
 * Пример SignatureExample предназначен
 * для выполнения подписи/проверки.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class SignatureExample implements Executable<Void> {

    /**
     * Данные для подписи.
     */
    public static final byte[] DATA = "test".getBytes();

    /**
     * Закрытый ключ для подписи.
     */
    private final PrivateKey privateKey;

    /**
     * Сертификат для проверки подписи.
     */
    private final X509Certificate cert;

    /**
     * Конструктор.
     *
     * @param privateKey Закрытый ключ для подписи.
     * @param cert Сертификат для проверки подписи.
     */
    public SignatureExample(PrivateKey privateKey, X509Certificate cert) {
        this.privateKey = privateKey;
        this.cert = cert;
    }

    @Override
    public Void execute() throws Exception {

        Logger.INSTANCE.append("Initializing signature: "
            + SIG_ALGORITHM + "...");

        Signature signature = Signature.getInstance(
            SIG_ALGORITHM, JCSP.PROVIDER_NAME);

        Logger.INSTANCE.append("Adding private key...");
        signature.initSign(privateKey);

        signature.update(DATA);
        byte[] sign = signature.sign();

        Logger.INSTANCE.append("Signature completed: " + sign.length
            + " bytes. Initializing verifying: " + SIG_ALGORITHM + "...");

        signature = Signature.getInstance(
            SIG_ALGORITHM, JCSP.PROVIDER_NAME);

        Logger.INSTANCE.append("Adding public key...");
        signature.initVerify(cert.getPublicKey());

        signature.update(DATA);
        boolean verified = signature.verify(sign);

        Logger.INSTANCE.append("Signature verified: "
            + verified);

        return null;

    }

}
