/**
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package oldSamples;

import java.io.*;
import java.security.KeyStore;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Principal;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.*;


public class ImportCert {
	public static void main(String[] args) {
        long startTime = General.printTestName("ImportCert");
        try {
 	    String keyAlias = "mykey";
	    String password = "password";
            String typeCert = "X509";
            String filename = "certnew.p7b";
            KeyStore keyStore = General.LoadKeyStore(password);
            CertificateFactory cf = CertificateFactory.getInstance(typeCert);

	    InputStream inStream = System.in;
	    inStream = new FileInputStream(filename);

	    PrivateKey privKey = General.GetPrivateFromStore(keyStore, keyAlias, password);
            Certificate userCert = keyStore.getCertificate(keyAlias);
	    if (userCert == null) {
	        throw new Exception("alias has no public key (certificate)");
	    }

	    Collection c = cf.generateCertificates(inStream);
	    if (c.isEmpty()) {
	        throw new Exception("Reply has no certificates");
	    }
	    Certificate[] replyCerts =
	    (Certificate[]) c.toArray (new Certificate[c.size()]);
	    Certificate[] newChain;
	    if (replyCerts.length == 1) {
 		    newChain = establishCertChain(userCert, replyCerts[0], keyStore);
	    }
	    else {
	        newChain = validateReply(userCert, replyCerts);
	    }

	    if (newChain != null) {
	        keyStore.setKeyEntry(keyAlias, privKey, password.toCharArray(), newChain);
	        General.SaveKeyStore(keyStore, password);
            System.out.println("Certificate reply was installed in keystore");
	        General.printOK(startTime);
            }
        else {
	        throw new Exception("Certificate reply was not installed in keystore");
	    }
    }
	catch( Exception ex )
	{
		General.printEx(ex, startTime);
	}
 }	

	public static Certificate[] validateReply(Certificate userCert,
					Certificate[] replyCerts)
	throws Exception
    {
	// order the certs in the reply (bottom-up).
	// we know that all certs in the reply are of type X.509, because
	// we parsed them using an X.509 certificate factory
	int i;
	PublicKey userPubKey = userCert.getPublicKey();
	for (i=0; i<replyCerts.length; i++) {
	    if (userPubKey.equals(replyCerts[i].getPublicKey()))
		break;
	}
	if (i == replyCerts.length) {
	    throw new Exception("Certificate reply does not contain public key for <alias>");
	}

	Certificate tmpCert = replyCerts[0];
	replyCerts[0] = replyCerts[i];
	replyCerts[i] = tmpCert;
	Principal issuer = ((X509Certificate)replyCerts[0]).getIssuerDN();

	for (i=1; i < replyCerts.length-1; i++) {
	    // find a cert in the reply whose "subject" is the same as the
	    // given "issuer"
	    int j;
	    for (j=i; j<replyCerts.length; j++) {
		Principal subject;
		subject = ((X509Certificate)replyCerts[j]).getSubjectDN();
		if (subject.equals(issuer)) {
		    tmpCert = replyCerts[i];
		    replyCerts[i] = replyCerts[j];
		    replyCerts[j] = tmpCert;
		    issuer = ((X509Certificate)replyCerts[i]).getIssuerDN();
		    break;
		}		
	    }
	    if (j == replyCerts.length) {
		throw new Exception ("Incomplete certificate chain in reply");
	    }
	}
	
	// now verify each cert in the ordered chain
	for (i=0; i<replyCerts.length-1; i++) {
	    PublicKey pubKey = replyCerts[i+1].getPublicKey();
	    try {
		replyCerts[i].verify(pubKey);
	    } catch (Exception e) {
		throw new Exception("Certificate chain in reply does not verify: " +
			e.getMessage());
	    }
	}
	return replyCerts;
    }


 public static Certificate[] establishCertChain(Certificate userCert,
					     Certificate certToVerify, KeyStore keyStore)
	throws Exception
    {
	if (userCert != null) {
	    // Make sure that the public key of the certificate reply matches
	    // the original public key in the keystore
	    PublicKey origPubKey = userCert.getPublicKey();
	    PublicKey replyPubKey = certToVerify.getPublicKey();
	    if (!origPubKey.equals(replyPubKey)) {
		throw new Exception("Public keys in reply and keystore don't match");
	    }

	    // If the two certs are identical, we're done: no need to import
	    // anything
	    if (certToVerify.equals(userCert)) {
		throw new Exception("Certificate reply and certificate in keystore are identical");
	    }
	}

	// Build a hash table of all certificates in the keystore.
	// Use the subject distinguished name as the key into the hash table.
	// All certificates associated with the same subject distinguished
	// name are stored in the same hash table entry as a vector.
	Hashtable certs = null;
	if (keyStore.size() > 0) {
	    certs = new Hashtable(11);
	    keystorecerts2Hashtable(keyStore, certs);
	}

	// start building chain
	Vector chain = new Vector(2);
	if (buildChain((X509Certificate)certToVerify, chain, certs)) {
	    Certificate[] newChain = new Certificate[chain.size()];
	    // buildChain() returns chain with self-signed root-cert first and
	    // user-cert last, so we need to invert the chain before we store
	    // it
	    int j=0;
	    for (int i=chain.size()-1; i>=0; i--) {
		newChain[j] = (Certificate)chain.elementAt(i);
		j++;
	    }
	    return newChain;
	} else {
	    throw new Exception("Failed to establish chain from reply");
	}
    }


 static public boolean buildChain(X509Certificate certToVerify, Vector chain,
			       Hashtable certs) {
	Principal subject = certToVerify.getSubjectDN();
	Principal issuer = certToVerify.getIssuerDN();	
	if (subject.equals(issuer)) {
	    // reached self-signed root cert;
	    // no verification needed because it's trusted.
	    chain.addElement(certToVerify);
	    return true;
	}

	// Get the issuer's certificate(s)
	Vector vec = (Vector)certs.get(issuer);
	if (vec == null)
	    return false;
	
	// Try out each certificate in the vector, until we find one
	// whose public key verifies the signature of the certificate
	// in question.
	for (Enumeration issuerCerts = vec.elements();
	     issuerCerts.hasMoreElements(); ) {
	    X509Certificate issuerCert
		= (X509Certificate)issuerCerts.nextElement();
	    PublicKey issuerPubKey = issuerCert.getPublicKey();
	    try {
		certToVerify.verify(issuerPubKey);
	    } catch(Exception e) {
		continue;
	    }
	    if (buildChain(issuerCert, chain, certs)) {
		chain.addElement(certToVerify);
		return true;
	    }
	}
	return false;
    }

 static public void keystorecerts2Hashtable(KeyStore ks, Hashtable hash)
	throws Exception
    {
	for (Enumeration aliases = ks.aliases(); aliases.hasMoreElements(); ) {
	    String alias = (String)aliases.nextElement();
	    Certificate cert = ks.getCertificate(alias);
	    if (cert != null) {
		Principal subjectDN = ((X509Certificate)cert).getSubjectDN();
		Vector vec = (Vector)hash.get(subjectDN);
		if (vec == null) {
		    vec = new Vector();
		    vec.addElement(cert);
		} else {
		    if (!vec.contains(cert))
			vec.addElement(cert);
		}
		hash.put(subjectDN, vec);
	    }
	}
    }

}