/**
 * $RCSfileICAConfiguration.java,v $
 * version $Revision: 36379 $
 * created 06.07.2015 10:22 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2015 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package userSamples.ca15;

import ru.CryptoPro.JCP.JCP;

/**
 * Класс используется для инициализации параметров
 * подключения в примерах.
 *
 * @author Copyright 2004-2015 Crypto-Pro. All rights reserved.
 * @.Version
 */
public abstract class CAConfiguration implements Configuration {

    /**
     * Текущая папка.
     */
    private static final String CURRENT_DIR = System.getProperty("user.dir");

    /**
     * Хранилище доверенных корневых сертификатов.
     */
    public static final String TRUST_STORE = CURRENT_DIR +
        "/data/KEYS/local_ca/CA15TrustedStore.store";

    /**
     * Пароль для доступа к хранилищу доверенных
     * корневых сертификатов.
     */
    public static final String TRUST_STORE_PASSWORD = "1";

    /**
     * Флаг инициализации.
     */
    private static boolean initiated = false;

    /**
     * Конструктор.
     *
     * @param needInit Флаг необходимости выполнить
     * инициализацию.
     */
    protected CAConfiguration(boolean needInit) {

        if (!initiated) {

            if (needInit) {
                init();
            } // if

        } // if

        initiated = true;

    }

    /**
     * Функция запуска примера.
     *
     * @throws Exception
     */
    public abstract void main() throws Exception;

    /**
     * Функция инициализации.
     *
     */
    private static void init() {

        // Включаем возможность онлайновой проверки.
        System.setProperty("com.sun.security.enableCRLDP", "true");
        System.setProperty("com.ibm.security.enableCRLDP", "true");

        // Настройки TLS, т.к. обращение к УЦ 2.0 будет выполняться
        // по протоколу HTTPS. Указываем тип хранилища с доверенным
        // корневым сертификатом, путь к нему и пароль.
        System.setProperty("javax.net.ssl.trustStoreType", JCP.CERT_STORE_NAME);
        System.setProperty("javax.net.ssl.trustStore", TRUST_STORE);
        System.setProperty("javax.net.ssl.trustStorePassword", TRUST_STORE_PASSWORD);

    }

}
